from typing import ContextManager
import pandas as pd

def obliterate_pandas() -> None: ...
def resurrect_pandas() -> None: ...
def missing_pandas() -> ContextManager[None]: ...
def is_pandas_available() -> bool: ...
def check_pandas() -> None: ...

__all__ = [
    "pd",
    "obliterate_pandas",
    "resurrect_pandas",
    "is_pandas_available",
    "check_pandas",
]



# =============================================================================
# Pelote Network to Tabular Conversion Functions
# =============================================================================
#
# Functions able to convert networkx to various tabular data formats.
#
from typing import Optional

from pelote.types import AnyGraph
from pelote.shim import pd, check_pandas


def to_nodes_dataframe(
    graph: AnyGraph, node_key_col: Optional[str] = "key"
) -> pd.DataFrame:
    check_pandas()

    if node_key_col is None:

        def raw_data():
            for _, a in graph.nodes(data=True):
                yield a

        return pd.DataFrame(data=raw_data(), index=list(graph.nodes))

    else:

        def data_with_key():
            for n, a in graph.nodes(data=True):
                r = {node_key_col: n}
                r.update(a)
                yield r

        return pd.DataFrame(data=data_with_key())



from setuptools import setup, find_packages

with open("./README.md", "r") as f:
    long_description = f.read()

setup(
    name="pelote",
    version="0.1.1",
    description="Collection of network-related utilities for python.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/medialab/pelote",
    license="MIT",
    author="Guillaume Plique",
    author_email="guillaume.plique@sciencespo.fr",
    keywords="network",
    python_requires=">=3.6",
    packages=find_packages(exclude=["test"]),
    package_data={"docs": ["README.md"], "pelote": ["*.pyi"]},
    install_requires=["networkx>=2,<3", "typing_extensions"],
    zip_safe=True,
)



# =============================================================================
# Pelote Utilities
# =============================================================================
#
# Miscellaneous utility functions used throughout the library.
#
from typing import Iterable, Any


def has_mixed_types(iterable: Iterable[Any]) -> bool:
    nodes = iter(iterable)

    main_type = None

    for node in nodes:
        main_type = type(node)
        break

    for node in nodes:
        if type(node) is not main_type:
            return True

    return False



import networkx as nx
from typing import Any, Union, TypeVar, Dict, List
from typing_extensions import TypedDict, Literal, NotRequired
from pathlib import Path
from io import IOBase

# Misc utils
FileHandle = Union[str, Path, IOBase]

# Networkx-related
NodeKey = TypeVar("NodeKey")

AnyGraph = Union[
    nx.Graph,
    nx.DiGraph,
    nx.MultiGraph,
    nx.MultiDiGraph,
]


# Graphology serialization
SerializedAttributes = Dict[str, Any]
GraphologyType = Literal["directed", "undirected", "mixed"]


class GraphologySerializedNode(TypedDict):
    key: str
    attributes: NotRequired[SerializedAttributes]


class GraphologySerializedEdge(TypedDict):
    key: NotRequired[str]
    source: str
    target: str
    attributes: NotRequired[SerializedAttributes]
    undirected: NotRequired[bool]


class GraphologySerializedOptions(TypedDict):
    allowSelfLoops: bool
    type: GraphologyType
    multi: bool


class GraphologySerializedGraph(TypedDict):
    options: NotRequired[GraphologySerializedOptions]
    attributes: NotRequired[SerializedAttributes]
    nodes: NotRequired[List[GraphologySerializedNode]]
    edges: NotRequired[List[GraphologySerializedEdge]]
